from baselib.py import httplib



class TestHttplib(object):

    def test_request(self):
        import ssl
        ssl._create_default_https_context = ssl._create_unverified_context
        conn = httplib.HTTPSConnection("www.python.org")
        conn.request("GET", "/index.html")
        r1 = conn.getresponse()
        print(r1.status, r1.reason)
        # data1 = r1.read()
        # print('data', data1)
        conn.close()


if __name__ == "__main__":
    TestHttplib().test_request()



# -*- coding:utf-8 -*-
import platform

__all__ = ['IS_PY3', 'httplib', 'urllib2']
IS_PY3 = int(platform.python_version_tuple()[0]) == 3

# 不需要用这些方式，可以用 from six.moves import urllib 
# ============================================================
# from baselib.py import httplib
if IS_PY3:
    import http.client as httplib
else:
    import httplib

# ============================================================
# from baselib.py import urllib2
"""
在Python3中，`urllib2`库已经被分拆成`urllib.request`和`urllib.error`等模块。
因此，在Python3中，您应该使用`urllib.request`代替`urllib2`。
"""
if IS_PY3:
    import urllib.request as urllib2
else:
    import urllib2


# -*- coding:utf-8 -*-
import logging
import logging.handlers
import sys
from os import makedirs
from os.path import dirname, exists


class settings:
    # 通用配置
    LOG_ENABLED = False  # 是否开启日志
    LOG_TO_CONSOLE = True  # 是否输出到控制台
    LOG_TO_FILE = True  # 是否输出到文件
    LOG_TO_ES = False  # 是否输出到 Elasticsearch
    LOG_LEVEL = 'DEBUG'  # 日志级别: ERROR/WARNING/INFO/DEBUG
    # LOG_FORMAT = '%(levelname)s - %(asctime)s - process: %(process)d -
    # %(filename)s - %(name)s - %(lineno)d - %(module)s - %(message)s'  #
    # 每条日志输出格式
    LOG_FORMAT = '[%(asctime)s][%(filename)s:%(lineno)d-%(module)s] - %(levelname)s - %(message)s'
    # 文件输出配置
    LOG_PATH = '/var/logs/runtime.log'  # 日志文件路径
    TIMED_ROTATING = True  # 日志是否按时间切割
    ROTATING_TIME = 'D'  # ,
    # ES输出日志
    ELASTIC_SEARCH_HOST = 'eshost'  # Elasticsearch Host
    ELASTIC_SEARCH_PORT = 9200  # Elasticsearch Port
    ELASTIC_SEARCH_INDEX = 'runtime'  # Elasticsearch Index Name
    APP_ENVIRONMENT = 'dev'  # 运行环境，如测试环境还是生产环境

loggers = {}


def file_logger(name=None, log_level="DEBUG", log_format=None, time_rotating='D', encoding='utf-8'):
    """
    param string name: 
    param string log_level: 日志级别: ERROR/WARNING/INFO/DEBUG
    param string log_format: 每条日志输出格式
    param string time_rotating: 日志切割时间, 可选值: S(秒)/M(分)/H(小时)/D(天)/W(周)/midnight(深夜)
    param string encoding: 编码
    """
    if not name:
        name = __name__
    logger = logging.getLogger(name)
    logger.setLevel(log_level.upper())
    if log_format is None:
        log_format = '[%(asctime)s][%(filename)s:%(lineno)d-%(module)s] - %(levelname)s - %(message)s'

    formatter = logging.Formatter(log_format)
    if time_rotating :
        file_handler = logging.handlers.TimedRotatingFileHandler(
        settings.LOG_PATH, when=time_rotating, encoding=encoding)
    else:
        file_handler = logging.FileHandler(settings.LOG_PATH, encoding=encoding)
    file_handler.setLevel(level=settings.LOG_LEVEL)
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)
    return logger

def get_logger(name=None):
    """
    get logger by name
    :param name: name of logger
    :return: logger
    """
    global loggers
    if not name:
        name = __name__
    if loggers.get(name):
        return loggers.get(name)
    logger = logging.getLogger(name)
    logger.setLevel(settings.LOG_LEVEL)
    formatter = logging.Formatter(settings.LOG_FORMAT)
    # 输出到控制台
    if settings.LOG_ENABLED and settings.LOG_TO_CONSOLE:
        stream_handler = logging.StreamHandler(sys.stdout)
        stream_handler.setLevel(level=settings.LOG_LEVEL)
        stream_handler.setFormatter(formatter)
        logger.addHandler(stream_handler)
    # 输出到文件
    if settings.LOG_ENABLED and settings.LOG_TO_FILE:
        # 如果路径不存在，创建日志文件文件夹
        log_dir = dirname(settings.LOG_PATH)
        if not exists(log_dir):
            makedirs(log_dir)
        # 添加 FileHandler
        if settings.TIMED_ROTATING:
            file_handler = logging.handlers.TimedRotatingFileHandler(
                settings.LOG_PATH, when=settings.ROTATING_TIME, encoding='utf-8')
        else:
            file_handler = logging.FileHandler(settings.LOG_PATH, encoding='utf-8')
        file_handler.setLevel(level=settings.LOG_LEVEL)
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)


    # 输出到 Elasticsearch
    if settings.LOG_ENABLED and settings.LOG_TO_ES:
        from cmreslogging.handlers import CMRESHandler
        # 添加 CMRESHandler
        es_handler = CMRESHandler(hosts=[{'host': settings.ELASTIC_SEARCH_HOST, 'port': settings.ELASTIC_SEARCH_PORT}],
                                  # 可以配置对应的认证权限
                                  auth_type=CMRESHandler.AuthType.NO_AUTH,
                                  es_index_name=settings.ELASTIC_SEARCH_INDEX,
                                  # 一个月分一个 Index
                                  index_name_frequency=CMRESHandler.IndexNameFrequency.MONTHLY,
                                  # 额外增加环境标识
                                  es_additional_fields={
                                      'environment': settings.APP_ENVIRONMENT}
                                  )
        es_handler.setLevel(level=settings.LOG_LEVEL)
        es_handler.setFormatter(formatter)
        logger.addHandler(es_handler)
    # 保存到全局 loggers
    loggers[name] = logger
    return logger


logger = get_logger()
if __name__ == "__main__":
    logger = get_logger()
    logger.debug("test")
    logger.warning("this is a warning message")
    logger.error("this is a error message")
    logger.info("this is a info message")



import os
from baselib.json import JsonFile


class TestJsonFile(object):

    def test_json_file(self):
        if os.path.isfile("test.json"):
            os.remove("test.json")
        data = JsonFile.read("test.json")
        assert data == {}
        JsonFile.write("test.json", {"msg": "hello"})
        data = JsonFile.read("test.json")
        assert data == {"msg": "hello"}
        JsonFile.update("test.json", {"msg": "world"})
        data = JsonFile.read("test.json")
        assert data == {"msg": "world"}
        JsonFile.update("test.json", {"info": "hello"})
        data = JsonFile.read("test.json")
        assert data == {"info": "hello", "msg": "world"}
        os.remove("test.json")
        with open("test.json", "w") as fid:
            fid.write("test")
        data = JsonFile.read("test.json")
        assert data == {}



# -*- coding:utf-8 -*-
# from distutils.core import setup
from setuptools import find_packages
from setuptools import setup


def read_desc(filename="README.md"):
    try:
        from pathlib import Path
        desc = Path(filename).read_text(encoding="utf-8")
    except BaseException:
        with open(filename, "r") as fid:
            desc = fid.read()
    return desc


setup(
    name='baselib',
    version="1.1.0.1",
    description='base common lib for python',
    long_description=read_desc(),
    long_description_content_type="text/markdown",
    author='coreylam',
    author_email='coreylam@163.com',
    url='https://github.com/coreylam/base_common_lib',
    license='GPL',
    include_package_data=True,
    packages=find_packages(),
    zip_safe=False,
    install_requires=[],
    python_requires=">=2.7",
)



# -*- coding:utf-8 -*-
from baselib.log import file_logger
import logging


class TestFileLogger:
    test_obj = file_logger()

    def test_default_log_level(self):
        assert self.test_obj.level == logging.DEBUG

    def test_default_log_format(self):
        formatter = logging.Formatter(
            '[%(asctime)s][%(levelname)s][%(filename)s:%(lineno)d-%(module)s] %(message)s')
        assert self.test_obj.handlers[0].formatter._fmt == formatter._fmt

    def test_default_time_rotating(self):
        assert isinstance(self.test_obj.handlers[0], logging.FileHandler)

    def test_no_rotating(self):
        test_obj = file_logger(time_rotating=None)
        test_obj.info("no time rotating")

    def test_custom_name(self):
        logger = file_logger(name='test')
        assert logger.name == 'test'

    def test_custom_log_level(self):
        logger = file_logger(log_level='WARNING')
        assert logger.level == logging.WARNING

    # def test_custom_log_format(self):
    #     log_format = '[%(asctime)s] %(message)s'
    #     logger = file_logger(log_format=log_format)
    #     formatter = logging.Formatter(log_format)
    #     assert logger.handlers[0].formatter._fmt == formatter._fmt

    # def test_time_rotating_Daily_rotation_interval(self):
    #     handler_class, interval, backup_count, delay, utc, at_time, encoding \
    #         = type(file_handler), 'D', 0, False, False, None, 'utf-8'
    #     handler_instance = handler_class(
    #         filename=log_path,
    #         when=interval,
    #         backupCount=backup_count,
    #         delay=delay,
    #         utc=utc,
    #         atTime=at_time,
    #         encoding=encoding
    #     )
    #     assert isinstance(
    #         handler_instance,
    #         TimedRotatingFileHandler) and handler_instance.when == 'D'



# -*- coding:utf-8 -*-
""" json 文件读写与更新
"""
import os
import json


class JsonFile(object):

    @classmethod
    def read(cls, file_path):
        """ 读取 文件路径为 $file_path 的 json 文件
        :param file_path: 文件路径，包括目录，名称，后缀
        :return:
            - 正常情况下，返回 json 文件
            - json 文件不存在，读取失败，以及内容为空时，返回 {}
        """
        if not os.path.isfile(file_path):
            return {}
        try:
            with open(file_path, 'r') as fid:
                data = json.load(fid)
        except Exception:
            return {}
        return data

    @classmethod
    def write(cls, file_path, data, indent=None):
        """ 写入 文件路径为 $file_path 的 json 文件
        :param string file_path: 文件路径，包括目录，名称，后缀
        :param dict data: 要写入的数据
        :param int indent: json 格式对齐，默认为None，不做格式化
        :return: 实际写入的数据（即入参 data）
        """
        with open(file_path, 'w') as fid:
            fid.writelines(json.dumps(data, indent=indent))
        return data

    @classmethod
    def update(cls, file_path, data, indent=None):
        """
        :param string file_path: 文件路径，包括目录，名称，后缀
        :param dict data: 要写入的数据
        :param int indent: json 格式对齐，默认为None，不做格式化
        :return: 实际写入的数据（即入参 data）
        """
        org_data = cls.read(file_path)
        org_data.update(data)
        return cls.write(file_path, org_data, indent)



# -*- coding:utf-8 -*-
import os
import subprocess
import sys
import signal

__all__  = ["mkdirs", "check_file", "run_cmd"]

class TimeoutExpired(Exception):
    pass


def alarm_handler(signum, frame):
    raise TimeoutExpired

def mkdirs(file_path):
    """
    判断文件路径所在的目录是否存在，如果不存在则创建目录
    :param file_path: 文件路径
    """
    directory = os.path.dirname(file_path)
    if not os.path.exists(directory):
        os.makedirs(directory)


def check_file(path):
    """
    This function checks if a file path exists and returns the type of the path.

    :param path: The path to the file.
    :type path: str
    :return: A string indicating the type of path. Possible values are 'File', 'Directory', or ''.
    :rtype: str
    """
    if os.path.exists(path):
        if os.path.isfile(path):
            return "File"
        elif os.path.isdir(path):
            return "Directory"
    else:
        return "NotExists"

def run_cmd(
        cmd,
        deamon=False,
        timeout=None,
        return_list=True,
        encoding="utf-8"):
    """
    执行系统命令, 并获取返回值

    :param cmd: str, 要执行的命令
    :param deamon: bool, 是否以守护进程的方式运行命令
    :param timeout: float, 命令执行的最长时间
    :param return_list: bool, 是否返回list类型的结果
    :param encoding: str, 命令执行结果的编码方式，默认为utf-8
    :return: list/str/Popen, 返回命令执行结果，如果deamon=True，则返回Popen对象
    """
    if sys.version_info.major == 2:
        subp = subprocess.Popen(['/bin/sh', '-c', cmd],
                                stdin=subprocess.PIPE,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE)
    else:
        subp = subprocess.Popen(['/bin/sh', '-c', cmd],
                                stdin=subprocess.PIPE,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE,
                                encoding=encoding)
    if deamon:
        return subp
    out = communicate(subp, timeout)
    if return_list:
        return out[0].split("\n")
    # out format: (stdout_data, stderr_data)
    return out


def communicate(subp, timeout=None):
    if sys.version_info[0] >= 3:
        expired = False
        try:
            return subp.communicate(timeout=timeout)
        except subprocess.TimeoutExpired:
            expired = True
        if expired:
            raise TimeoutExpired("Command timed out after {} seconds".format(timeout))

    # Set up a SIGALRM handler to raise an exception after the specified timeout
    if timeout is not None:
        signal.signal(signal.SIGALRM, alarm_handler)
        signal.alarm(timeout)
    try:
        # Write the input data to stdin and read the output from stdout and stderr
        out, err = subp.communicate()
    except TimeoutExpired:
        # If the timeout was reached, kill the child process and raise a TimeoutError
        subp.kill()
        raise TimeoutExpired("Command timed out after {} seconds".format(timeout))
    finally:
        # Reset the SIGALRM handler to its default behavior
        signal.signal(signal.SIGALRM, signal.SIG_DFL)

    # Return the output and error as strings
    return out.decode(), err.decode()



# -*- coding:utf-8 -*-
""" 时间段计算，用于获取某个时间段的起止时间（如1小时内，24小时内，7天内等）
"""
import datetime
import time


class Period(object):

    @classmethod
    def get_recent_hour(cls, n, minutes_ago=0):
        """ 截止到 minutes_ago 前，获取近 n 个小时的起止时间
        @param n: n 个小时到截止时间
        @param minutes_ago: 截止统计时间，默认为 0， 表示截止到当前位置
        """
        return cls.get_recent_minute(n * 60, minutes_ago)

    @classmethod
    def get_recent_minute(cls, n, minutes_ago=0):
        """ 截止到 minutes_ago 前，获取近 n 分钟的起止时间
        @param n: int, 到截止时间之前 n 分钟
        @param minutes_ago: int, 截止统计时间，默认为 0， 表示截止到当前位置
        """
        start_time = (datetime.datetime.now() -
                      datetime.timedelta(minutes=n + minutes_ago)).replace(microsecond=0)
        end_time = (datetime.datetime.now() -
                    datetime.timedelta(minutes=minutes_ago)).replace(microsecond=0)
        return (start_time, end_time)

    @classmethod
    def get_recent_day(cls, n, minutes_ago=0):
        """ 截止到 minutes_ago 前，获取近 n 分钟的起止时间
        @param n: int, 到截止时间之前 n 分钟
        @param minutes_ago: int, 截止统计时间，默认为 0， 表示截止到当前位置
        """
        start_time = (datetime.datetime.now() -
                      datetime.timedelta(days=n, minutes=minutes_ago)).replace(microsecond=0)
        end_time = (datetime.datetime.now() -
                    datetime.timedelta(minutes=minutes_ago)).replace(microsecond=0)
        return (start_time, end_time)

    @classmethod
    def get_day_begin(cls, dt=None):
        """ 获取指定某一天的起始时间，即当天的0时0分0秒
        @param dt: datetime， 指定的日期，默认为None，表示今天
        """
        if dt is None:
            dt = datetime.datetime.now()
        return dt.replace(hour=0, minute=0, second=0, microsecond=0)

    @classmethod
    def get_day_end(cls, dt=None):
        """ 获取指定某一天的结束时间，即当天的23时59分59秒
        @param dt: datetime， 指定的日期，默认为None，表示今天
        """
        if dt is None:
            dt = datetime.datetime.now()
        return dt.replace(hour=23, minute=59, second=59, microsecond=0)

    @classmethod
    def get_month_begin(cls, dt=None):
        """ 获取指定某一天所在月的起始时间，即当月的1号的0时0分0秒
        @param dt: datetime， 指定的日期，默认为None，表示今天
        """
        if dt is None:
            dt = datetime.datetime.now()
        return dt.replace(day=1, hour=0, minute=0, second=0, microsecond=0)

    @classmethod
    def get_month_end(cls, dt=None):
        """ 获取指定某一天所在月的结束时间，即当天所在月最后一天的23时59分59秒
        @param dt: datetime， 指定的日期，默认为None，表示今天
        """
        if dt is None:
            dt = datetime.datetime.now()
        # TODO: 待实现
        return dt.replace(hour=23, minute=59, second=59, microsecond=0)

    @classmethod
    def now(cls):
        """ 获取当前时间
        :rtype: datetime
        :return: 返回当前的时间（datetime格式）
        """
        return datetime.datetime.now()

    @classmethod
    def format_datetime(cls, dt):
        return dt.strftime("%Y-%m-%d %H:%M:%S")

    @classmethod
    def format_t_datetime(cls, dt):
        return dt.strftime("%Y-%m-%dT%H:%M:%SZ")

    @classmethod
    def format_date(cls, dt):
        return dt.strftime("%Y-%m-%d")

    @classmethod
    def format_time(cls, dt):
        return dt.strftime("%H:%M:%S")

    @classmethod
    def format_timestamp(cls, dt):
        """
        :param datetime dt:
        :rtype: int
        :return: dt 对应的时间戳
        """
        return int(time.mktime(dt.timetuple()))


if __name__ == "__main__":
    start_time, end_time = Period.get_recent_day(10, 10)
    print(start_time, end_time)
    print(Period.format_date(start_time))
    print(Period.format_time(start_time))
    print(Period.format_datetime(start_time))
    print(Period.format_t_datetime(start_time))
    print(1)
    print(Period.get_day_begin(start_time))
    print(Period.format_t_datetime(Period.get_day_end(end_time)))
